==> pyutmpx-0.2.2.tar.gz: pyutmpx-0.2.2/src/main.c <==
/* ****************************************************************************
 * main.c -- pyutmpx module definition.
 * Copyright (C) 2017-2018 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of the pyutmpx Python 3.x module, which is MIT-licensed.
 * ************************************************************************* */

#include "pyutmpx.h"
#include <datetime.h>

/* ---
 * Datetime utilities.
 * --- */

/* Utility to get a datetime from a timeval. */

PyObject *pyutmpx_get_datetime(const struct timeval *tv)
{
	PyObject *result = NULL;
	PyObject *epoch = NULL, *date_delta = NULL;

	epoch = PyDateTime_FromDateAndTime(1970, 1, 1, 0, 0, 0, 0);
	date_delta = PyDelta_FromDSU(0, tv->tv_sec, tv->tv_usec);
	if (!epoch || !date_delta)
		goto fail;

	result = PyNumber_Add(epoch, date_delta);

fail:
	Py_XDECREF(epoch);
	Py_XDECREF(date_delta);

	return (result);
}

/* ---
 * Module definitions.
 * --- */

/* Module deinitialization.
 * We ought to deinitialize objects that the user can't see. */

static void pyutmpx_del(void *module)
{
	(void)module;

	pyutmpx_destroy_iterators();
}

/* Module methods. */

static PyMethodDef module_methods[] = {
	{NULL, NULL, 0, NULL}
};

/* Module definition. */

static struct PyModuleDef module = {
	PyModuleDef_HEAD_INIT,

	/* .m_name = */ "pyutmpx",
	/* .m_doc = */ "This module provides a utmp reader.",
	/* .m_size = */ -1,
	/* .m_methods = */ module_methods,
	/* .m_slots = */ NULL,
	/* .m_traverse = */ NULL,
	/* .m_clear = */ NULL,
	/* .m_free = */ &pyutmpx_del
};

/* Module initialization. */

PyMODINIT_FUNC PyInit_pyutmpx(void)
{
	PyObject *m;
	int p_iter = 0;
	int p_types = 0;

	/* Import the datetime module. */

	PyDateTime_IMPORT;

	/* Initialize the module. */

	m = PyModule_Create(&module);
	if (!m)
		goto fail;

	/* Version. */

	if (PyModule_AddStringConstant(m, "version", PYUTMPX_VERSION) < 0)
		goto fail;

	/* Prepare the iterators. */

	if (pyutmpx_prepare_iterators(m))
		goto fail;
	p_iter = 1;

	/* Make the constants and add them to the module. */

	if (pyutmpx_prepare_types(m))
		goto fail;
	p_types = 1;

	/* Create the utmp entry type to the module. */

	if (PyType_Ready(&pyutmpx_entry) < 0)
		goto fail;
	Py_INCREF((PyObject *)&pyutmpx_entry);
	if (PyModule_AddObject(m, "utmp_entry", (PyObject *)&pyutmpx_entry) < 0) {
		Py_DECREF(&pyutmpx_entry);
		goto fail;
	}

	/* Everything went well in the end :) */

	return (m);
fail:
	if (p_types)
		pyutmpx_destroy_types();
	if (p_iter)
		pyutmpx_destroy_iterators();
	Py_XDECREF(m);
	return (NULL);
}

==> pyutmpx-0.4.tar.gz: pyutmpx-0.4/src/main.c <==
/* ****************************************************************************
 * main.c -- pyutmpx module definition.
 * Copyright (C) 2017-2021 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of the pyutmpx Python 3.x module, which is MIT-licensed.
 * ************************************************************************* */

#include "pyutmpx.h"

/* ---
 * Module definition.
 * --- */

/* Module deinitialization.
 * We ought to deinitialize objects that the user can't see. */

static void del_module(void *module)
{
	pyutmpx_exit_lastlog_type();
	pyutmpx_exit_lastlog_entry_type();
	pyutmpx_exit_btmp_type();
	pyutmpx_exit_wtmp_type();
	pyutmpx_exit_utmp_type();
	pyutmpx_exit_utmp_entry_type();
	pyutmpx_exit_exit_status_type();
	pyutmpx_exit_utils();
}

/* Module methods. */

static PyMethodDef module_methods[] = {
	{NULL, NULL, 0, NULL}
};

/* Module definition. */

static struct PyModuleDef module = {
	PyModuleDef_HEAD_INIT,

	/* .m_name = */ "pyutmpx",
	/* .m_doc = */ "This module provides a utmp reader.",
	/* .m_size = */ -1,
	/* .m_methods = */ module_methods,
	/* .m_slots = */ NULL,
	/* .m_traverse = */ NULL,
	/* .m_clear = */ NULL,
	/* .m_free = */ &del_module
};

/* ---
 * Module initialization.
 * --- */

PyMODINIT_FUNC PyInit_pyutmpx(void)
{
	PyObject *m;

	/* Initialize the module. */

	m = PyModule_Create(&module);
	if (!m)
		goto fail;

	/* Version. */

	if (PyModule_AddStringConstant(m, "version", PYUTMPX_VERSION) < 0)
		goto fail;

	/* Setup the various modules. */

	if (pyutmpx_init_utils(m))
		goto fail;
	if (pyutmpx_init_exit_status_type(m))
		goto fail;
	if (pyutmpx_init_utmp_entry_type(m))
		goto fail;
	if (pyutmpx_init_utmp_type(m))
		goto fail;
	if (pyutmpx_init_wtmp_type(m))
		goto fail;
	if (pyutmpx_init_btmp_type(m))
		goto fail;
	if (pyutmpx_init_lastlog_entry_type(m))
		goto fail;
	if (pyutmpx_init_lastlog_type(m))
		goto fail;

	/* Everything went well in the end :) */

	return (m);
fail:
	Py_XDECREF(m);
	return (NULL);
}
